* SMPL-AD.ASM	5/8/96

* Simple Serial A/D board driver FOR CMD11 AND CMM11...
* Runs under the buffalo monitor for terminal I/O
*
* Operation - program configures SPI port (SS:KEYBOARD)
*	for master out mode and scans SS-AD board channels
*	as configured in the SCAN_TABLE.  Channel data is
*	printed back to the SCI port with the Buffalo Monitor's
*	support routines. 
*
* EXECUTION STEPS:
* 1. Load program under buffalo with LOAD T <CR command.
* 2. Start program with CALL 3000 <CR command.
* 3. Program will columnize 8 channels of data across terminal
*	and scroll.
* 4. To end program press any key on terminal keyboard. 


RAM	EQU	$2000	* DEFINE RAM START ADDRESS (VARIABLE LOCATION)
ROM	EQU	$3000	* DEFINE ROM START ADDRESS (PROGRAM LOCATION)

* 68HC11A1 Register definitions...

REGS         EQU        $1000		* Default reset address
PortA        EQU        REGS+$00
PortC        EQU        REGS+$03
PortB        EQU        REGS+$04
DDRC         EQU        REGS+$07
PortD        EQU        REGS+$08
DDRD         EQU        REGS+$09
PortE        EQU        REGS+$0A
CFORC        EQU        REGS+$0B
TCNT         EQU        REGS+$0E        * WORD
TIC1         EQU        REGS+$10        * WORD
TIC2         EQU        REGS+$12        * WORD
TIC3         EQU        REGS+$14        * WORD
TOC1         EQU        REGS+$16        * WORD
TOC2         EQU        REGS+$18        * WORD
TOC3         EQU        REGS+$1A        * WORD
TOC4         EQU        REGS+$1C        * WORD
TOC5         EQU        REGS+$1E        * WORD
TCTL1        EQU        REGS+$20
TCTL2        EQU        REGS+$21
TMSK1        EQU        REGS+$22
TFLG1        EQU        REGS+$23
TMSK2        EQU        REGS+$24
TFLG2        EQU        REGS+$25
PACTL        EQU        REGS+$26
PACNT        EQU        REGS+$27
SPCR	     EQU	REGS+$28
SPSR	     EQU	REGS+$29
SPDR	     EQU	REGS+$2A
BAUD         EQU        REGS+$2B
SCCR1        EQU        REGS+$2C
SCCR2        EQU        REGS+$2D
SCSR         EQU        REGS+$2E
SCDR         EQU        REGS+$2F
ADCTL        EQU        REGS+$30
OPTION       EQU        REGS+$39
PPROG        EQU        REGS+$3B
HPRIO        EQU        REGS+$3C
INIT         EQU        REGS+$3D
TEST         EQU        REGS+$3E
CONFIG       EQU        REGS+$3F

* Simple Serial driver definitions...

SS_port EQU     PortD   ; define simple serial port

* Simple Serial port bit definitions...

SS_DIN  equ     $04     * Data input bit 
SS_DOUT equ     $08     * Data output bit
SS_CLK  equ     $10     * Data clock bit
SS_CS1  equ     $20     * Chip select 1 out

* Simple Serial Analog to Digital Converter Select

A2D_CS	equ	SS_CS1	* Define chip select

*....................................................................

* RAM DEFINTIONS

* Simple Serial A/D converter Ram and Registers...

AD_ptr		equ	RAM		* A/D Scan table pointer
AD_hi		equ	AD_ptr+2	* A/D hi 8 bits of 12 bit result
AD_lo		equ	AD_hi+1		* A/D lo 4 bits in upper nibble
AD_mode		equ	AD_lo+1		* A/D current mode and channel control

* A/D converter command definitions...

SGL_UN_0	equ	$8f	; Single channel 0 input, unipolar
SGL_UN_1	equ	$cf	; Single channel 1 input, unipolar
SGL_UN_2	equ	$9f	; Single channel 2 input, unipolar
SGL_UN_3	equ	$df	; Single channel 3 input, unipolar
SGL_UN_4	equ	$af	; Single channel 4 input, unipolar
SGL_UN_5	equ	$ef	; Single channel 5 input, unipolar
SGL_UN_6	equ	$bf	; Single channel 6 input, unipolar
SGL_UN_7	equ	$ff	; Single channel 7 input, unipolar

SGL_BI_0	equ	$87	; Single channel 0 input, bipolar
SGL_BI_1	equ	$c7	; Single channel 1 input, bipolar
SGL_BI_2	equ	$97	; Single channel 2 input, bipolar
SGL_BI_3	equ	$d7	; Single channel 3 input, bipolar
SGL_BI_4	equ	$a7	; Single channel 4 input, bipolar
SGL_BI_5	equ	$e7	; Single channel 5 input, bipolar
SGL_BI_6	equ	$b7	; Single channel 6 input, bipolar
SGL_BI_7	equ	$f7	; Single channel 7 input, bipolar

DIF_OD_UN_0	equ	$4f	; Differential 0-|1+ input, unipolar
DIF_OD_UN_1	equ	$5f	; Differential 0-|1+ input, unipolar
DIF_OD_UN_2	equ	$6f	; Differential 0-|1+ input, unipolar
DIF_OD_UN_3	equ	$7f	; Differential 0-|1+ input, unipolar

DIF_SI_UN_0	equ	$0f	; Differential 0+|1- input, unipolar
DIF_SI_UN_1	equ	$1f	; Differential 0+|1- input, unipolar
DIF_SI_UN_2	equ	$2f	; Differential 0+|1- input, unipolar
DIF_SI_UN_3	equ	$3f	; Differential 0+|1- input, unipolar

DIF_OD_BI_0	equ	$47	; Differential 0-|1+ input, Bipolar
DIF_OD_BI_1	equ	$57	; Differential 0-|1+ input, Bipolar
DIF_OD_BI_2	equ	$67	; Differential 0-|1+ input, Bipolar
DIF_OD_BI_3	equ	$77	; Differential 0-|1+ input, Bipolar

DIF_SI_BI_0	equ	$07	; Differential 0+|1- input, Bipolar
DIF_SI_BI_1	equ	$17	; Differential 0+|1- input, Bipolar
DIF_SI_BI_2	equ	$27	; Differential 0+|1- input, Bipolar
DIF_SI_BI_3	equ	$37	; Differential 0+|1- input, Bipolar

* BUFFALO Support Routine entry addresses...

UPCASE      EQU         $FFA0   convert to upper case
WCHEK       EQU         $FFA3   check for white space
DCHEK       EQU         $FFA6   check for delimeter
* INIT        EQU               $FFA9   initialize i/o device
INPUT       EQU         $FFAC   low level input routine
* OUTPUT      EQU               $FFAF   low level output routine
OUTLHLF     EQU         $FFB2   display top 4 bits as hex digit
OUTRHLF     EQU         $FFB5   display bottom 4 bits as hex digit
OUTA        EQU         $FFB8   output ascii character in A
OUT1BYT     EQU         $FFBB   display the hex value of byte at X
OUT1BSP     EQU         $FFBE   out1byt followed by space
OUT2BSP     EQU         $FFC1   display 2 hex bytes at x and a space
OUTCRLF     EQU         $FFC4   carriage return, line feed to terminal
OUTSTRG     EQU         $FFC7   display string at X (term with $04)
OUTSTRG0    EQU         $FFCA   outstrg with no initial carr ret
INCHAR      EQU         $FFCD   wait for and input a char from term
			       
EOT     EQU     $04     end of a text table

****************************************************************************

* PROGRAM START....

	ORG	ROM

* Simple Serial Driver initialization...

SET_SS
	CLR	SS_port		* Clear the port...
	LDAA	#$3A		* Configure port direction
	STAA	DDRD
	LDAA	#$52		* Configure SPI port
	STAA	SPCR		* 

* Simple Serial A/D converter initialization...

SET_AD
	LDX	#SCAN_TABLE	* Set scan pointer
	STX	AD_ptr		* 
	JSR	AD_UPDATE	* Start first conversion

* Main program loop, Read A/D per Scan Table sequence

MAIN
	JSR     OUTCRLF         * LINE FEED TERMINAL, START NEW ROW
MAIN1
	JSR	AD_UPDATE	* Do next conversion
* 			use output time for conversion time
	LDX	#AD_hi		* Set TO channel high byte
	JSR     OUT2BSP         * PRINT HIGH AND LOW BYTE
* 			test for end of row, all channels read
	LDX	AD_ptr		* Test pointer for start of table
	CPX     #SCAN_TABLE+1   * 
	BNE     MAIN1           * If not, loop on row
* 			test for user exit...
	LDAA    SCSR		*
	ANDA    #$20            * TEST ANY USER INPUT
	BEQ     MAIN		* New row if not
	LDAA	SCDR		* CLEAR USER KEY
	RTS			* DONE

* Simple Serial Driver (SPI), source code...

SS_IO
	LDAB	SPSR		* CLEAR STATUS OF SPI
	STAA	SPDR		* SEND / RECEIVE BYTE
SS_IO_LP		
	LDAB	SPSR		* GET STATUS
	BPL	SS_IO_LP	* WAIT IF NO FLAG
* 			GET DATA AND QUIT
	LDAA	SPDR
	RTS

* Simple Serial Analog to Digital Converter Driver, source code...

AD_UPDATE
	LDX	AD_ptr		* Get scan table pointer
AD_UPD1
	LDAA	0,X		* Get the next mode
	BNE	AD_UPD2		* Go do it, if not end of table
* 			else, reset scan table pointer
	LDX	#SCAN_TABLE	* Load table start
	BRA	AD_UPD1		* Back to get table data
AD_UPD2
	INX			* up to next table positon
	STX	AD_ptr		* save pointer for next time
	LDX	#SS_port	* set pointer to select port
	BSET	0,X A2D_CS	* select A/D converter
	JSR	SS_IO		* send command, read hi byte
	STAA	AD_hi		* save hi byte
	CLRA			*
	JSR	SS_IO		* read low byte
	STAA	AD_lo		* save
	BCLR	0,X A2D_CS	* deselect and start next conversion
	RTS			* done


* A/D CHANNEL SCAN TABLE, EACH ENTRY = INPUT MODE FOR PARTICULAR A/D 
*	CHANNEL(S).  TABLE END IS EMPTY BYTE (=00).  TABLE CAN BE MIXED
*	WITH DIFFERENTIAL PAIRS, SINGLE CHANNELS, UNIPOLAR, AND BIPOLAR
*	COMMANDS FOR EACH A/D INPUT CONFIGURATION AND CONNECTION.

SCAN_TABLE

	FCB	SGL_UN_0,SGL_UN_1,SGL_UN_2,SGL_UN_3
	FCB	SGL_UN_4,SGL_UN_5,SGL_UN_6,SGL_UN_7
* TABLE END
	FCB	$00


	END

