* SMPL-DA.ASM	10/9/96

* Simple Serial D/A board driver FOR CMD11 AND CMM11...
* Runs under the buffalo monitor for terminal I/O
*
* Operation - program configures SPI port (SS:KEYBOARD)
*	for master out mode and sends SS-DA board channel
*	data as configured in the SCAN_TABLEA,B,C,D. Channel output
*	can be monitored with a Oscope to see updates
*
* EXECUTION STEPS:
* 1. Load program under buffalo with LOAD T <CR command.
* 2. Start program with CALL 3000 <CR command.
* 3. Program will stair step D/A channel outputs.
* 4. To end program press any key on terminal keyboard. 

RAM	EQU	$2000	* DEFINE RAM START ADDRESS (VARIABLE LOCATION)
ROM	EQU	$3000	* DEFINE ROM START ADDRESS (PROGRAM LOCATION)

* 68HC11A1 Register definitions...

REGS         EQU        $1000		* Default reset address
PortA        EQU        REGS+$00
PortC        EQU        REGS+$03
PortB        EQU        REGS+$04
DDRC         EQU        REGS+$07
PortD        EQU        REGS+$08
DDRD         EQU        REGS+$09
PortE        EQU        REGS+$0A
CFORC        EQU        REGS+$0B
TCNT         EQU        REGS+$0E        * WORD
TIC1         EQU        REGS+$10        * WORD
TIC2         EQU        REGS+$12        * WORD
TIC3         EQU        REGS+$14        * WORD
TOC1         EQU        REGS+$16        * WORD
TOC2         EQU        REGS+$18        * WORD
TOC3         EQU        REGS+$1A        * WORD
TOC4         EQU        REGS+$1C        * WORD
TOC5         EQU        REGS+$1E        * WORD
TCTL1        EQU        REGS+$20
TCTL2        EQU        REGS+$21
TMSK1        EQU        REGS+$22
TFLG1        EQU        REGS+$23
TMSK2        EQU        REGS+$24
TFLG2        EQU        REGS+$25
PACTL        EQU        REGS+$26
PACNT        EQU        REGS+$27
SPCR	     EQU	REGS+$28
SPSR	     EQU	REGS+$29
SPDR	     EQU	REGS+$2A
BAUD         EQU        REGS+$2B
SCCR1        EQU        REGS+$2C
SCCR2        EQU        REGS+$2D
SCSR         EQU        REGS+$2E
SCDR         EQU        REGS+$2F
ADCTL        EQU        REGS+$30
OPTION       EQU        REGS+$39
PPROG        EQU        REGS+$3B
HPRIO        EQU        REGS+$3C
INIT         EQU        REGS+$3D
TEST         EQU        REGS+$3E
CONFIG       EQU        REGS+$3F

* Simple Serial driver definitions...

SS_port EQU     PortD   ; define simple serial port

* Simple Serial port bit definitions...

SS_DIN  equ     $04     * Data input bit 
SS_DOUT equ     $08     * Data output bit
SS_CLK  equ     $10     * Data clock bit
SS_CS1  equ     $20     * Chip select 1 out

* Simple Serial Analog to Digital Converter Select

D2A_CS	equ	SS_CS1	* Define chip select

* DAC control command definitions... 

SET_DAC_A	equ	$1	; Set DAC channel A value, no load
SET_DAC_B	equ	$5	; Set DAC channel B value, no load
SET_DAC_C	equ	$9	; Set DAC channel C value, no load
SET_DAC_D	equ	$0d	; Set DAC channel D value, no load

LOAD_DAC_A	equ	$3	; Load DAC channel A with new value
LOAD_DAC_B	equ	$7	; Load DAC channel B with new value
LOAD_DAC_C	equ	$0b	; Load DAC channel C with new value
LOAD_DAC_D	equ	$0f	; Load DAC channel D with new value

LOAD_DAC_ALL	equ	$00	; Load all DACs with new value 
NOP_DAC		equ	$4	; No operation, allows shift of data
LDAC_DAC	equ	$2	; Load All DACS with individual data
MODE0_DAC	equ	$0a	; Dout clocked out on falling edge
MODE1_DAC	equ	$0e	; Dout clocked out on rising edge

*....................................................................

* RAM DEFINTIONS

* Digital to Analog Converter Output registers...

DAC_outA	equ	RAM	; output channel A table pointer
DAC_outB	equ	RAM+1	; output channel B table pointer
DAC_outC	equ	RAM+2	; output channel B table pointer
DAC_outD	equ	RAM+3	; output channel B table pointer

* BUFFALO Support Routine entry addresses...

UPCASE      EQU         $FFA0   convert to upper case
WCHEK       EQU         $FFA3   check for white space
DCHEK       EQU         $FFA6   check for delimeter
* INIT        EQU               $FFA9   initialize i/o device
INPUT       EQU         $FFAC   low level input routine
* OUTPUT      EQU               $FFAF   low level output routine
OUTLHLF     EQU         $FFB2   display top 4 bits as hex digit
OUTRHLF     EQU         $FFB5   display bottom 4 bits as hex digit
OUTA        EQU         $FFB8   output ascii character in A
OUT1BYT     EQU         $FFBB   display the hex value of byte at X
OUT1BSP     EQU         $FFBE   out1byt followed by space
OUT2BSP     EQU         $FFC1   display 2 hex bytes at x and a space
OUTCRLF     EQU         $FFC4   carriage return, line feed to terminal
OUTSTRG     EQU         $FFC7   display string at X (term with $04)
OUTSTRG0    EQU         $FFCA   outstrg with no initial carr ret
INCHAR      EQU         $FFCD   wait for and input a char from term
			       
EOT     EQU     $04     end of a text table

****************************************************************************

* PROGRAM START....

	ORG	ROM

* Simple Serial Driver initialization...

SET_SS
	CLR	SS_port		* Clear the port...
	LDAA	#$3A		* Configure port direction
	STAA	DDRD
	LDAA	#$52		* Configure SPI port
	STAA	SPCR		* 

* Simple Serial D/A converter initialization...

SET_DA
	ldx	#SS_port
	bset	0,x D2A_CS	* deselect DAC
	clr	DAC_outA	* clear outputs to 0 level
	clr	DAC_outB	* clear outputs to 0 level
	clr	DAC_outC	* clear outputs to 0 level
	clr	DAC_outD	* clear outputs to 0 level
	ldaa	#LOAD_DAC_ALL	* load DAC ALL command
	bclr	0,x D2A_CS  	* Select DAC
	jsr	SS_IO		* send command to load all with value
	clra			* clear all DACS
	jsr	SS_IO		* send data
	bset	0,x D2A_CS  	* deselect DAC, latch data

* Main program loop, send D/A data Scan Table sequence

MAIN
	LDX	#TABLE_A	* Point to table
	LDAB	DAC_outA	* get current step
	ABX			* compute step
	LDAB	0,x		* get step data
	BNE	MAIN1		* test if 0
	CLR	DAC_outA	* if yes, reset table scan
	BRA	MAIN		* read again
MAIN1
	INC	DAC_outA	* bump table pointer
	LDAA	#LOAD_DAC_A	* get command
	JSR	DA_UPDATE	* send 
MAIN2
	LDX	#TABLE_B	* Point to table
	LDAB	DAC_outB	* get current step
	ABX			* compute step
	LDAB	0,x		* get step data
	BNE	MAIN3		* test if 0
	CLR	DAC_outB	* if yes, reset table scan
	BRA	MAIN2		* read again
MAIN3
	INC	DAC_outB	* bump table pointer
	LDAA	#LOAD_DAC_B	* get command
	JSR	DA_UPDATE	* send 
MAIN4
	LDX	#TABLE_C	* Point to table
	LDAB	DAC_outC	* get current step
	ABX			* compute step
	LDAB	0,x		* get step data
	BNE	MAIN5		* test if 0
	CLR	DAC_outC	* if yes, reset table scan
	BRA	MAIN4		* read again
MAIN5
	INC	DAC_outC	* bump table pointer
	LDAA	#LOAD_DAC_C	* get command
	JSR	DA_UPDATE	* send 
MAIN6
	LDX	#TABLE_D	* Point to table
	LDAB	DAC_outD	* get current step
	ABX			* compute step
	LDAB	0,x		* get step data
	BNE	MAIN7		* test if 0
	CLR	DAC_outD	* if yes, reset table scan
	BRA	MAIN		* read again
MAIN7
	INC	DAC_outD	* bump table pointer
	LDAA	#LOAD_DAC_D	* get command
	JSR	DA_UPDATE	* send 

* check for user input

	LDAA    SCSR		*
	ANDA    #$20            * TEST ANY USER INPUT
	BEQ     MAIN		* New row if not
	LDAA	SCDR		* CLEAR USER KEY
	RTS			* DONE

* Simple Serial Driver (SPI), source code...

SS_IO
	LDAB	SPSR		* CLEAR STATUS OF SPI
	STAA	SPDR		* SEND / RECEIVE BYTE
SS_IO_LP		
	LDAB	SPSR		* GET STATUS
	BPL	SS_IO_LP	* WAIT IF NO FLAG
	RTS

* Simple Serial Analog to Digital Converter Driver, source code...

DA_UPDATE
	PSHB			* save data
	LDX	#SS_port	* set pointer to select port
	BCLR	0,X D2A_CS	* select A/D converter
	BSR	SS_IO		* send command
	PULA			* get data
	BSR	SS_IO		* send low byte
	BSET	0,X D2A_CS	* deselect and start next conversion
	RTS			* done


* D/A CHANNEL SCAN TABLEs

TABLE_A
	FCB	$2,$14,$21,$33,$45,$57,$69,$7B
	FCB	$8E,$9F,$AF,$B4,$C6,$D9,$EE,$F8
	FCB	$FF,$EE,$DD,$CC,$BB,$AA,$99,$88
	FCB	$77,$66,$55,$44,$33,$22,$11,$3
* TABLE END
	FCB	$00

TABLE_B
	FCB	$2,$14,$21,$33,$45,$57,$69,$7B
	FCB	$8E,$9F,$AF,$B4,$C6,$D9,$EE,$F8
	FCB	$FF,$EE,$DD,$CC,$BB,$AA,$99,$88
	FCB	$77,$66,$55,$44,$33,$22,$11,$3
* TABLE END
	FCB	$00

TABLE_C
	FCB	$2,$14,$21,$33,$45,$57,$69,$7B
	FCB	$8E,$9F,$AF,$B4,$C6,$D9,$EE,$F8
	FCB	$FF,$EE,$DD,$CC,$BB,$AA,$99,$88
	FCB	$77,$66,$55,$44,$33,$22,$11,$3
* TABLE END
	FCB	$00

TABLE_D
	FCB	$2,$14,$21,$33,$45,$57,$69,$7B
	FCB	$8E,$9F,$AF,$B4,$C6,$D9,$EE,$F8
	FCB	$FF,$EE,$DD,$CC,$BB,$AA,$99,$88
	FCB	$77,$66,$55,$44,$33,$22,$11,$3
* TABLE END
	FCB	$00

	END
