/* asm-m68hc12/ports_def.h -- Definition of 68HC12 ports
   Copyright 2000 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GDB, GAS, and the GNU binutils.

GDB, GAS, and the GNU binutils are free software; you can redistribute
them and/or modify them under the terms of the GNU General Public
License as published by the Free Software Foundation; either version
1, or (at your option) any later version.

GDB, GAS, and the GNU binutils are distributed in the hope that they
will be useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this file; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifndef _M68HC12_PORTS_DEF_H
#define _M68HC12_PORTS_DEF_H

/* Flags for the definition of the 68HC11 & 68HC12 CCR.  */
#define M6811_S_BIT     0x80	/* Stop disable */
#define M6811_X_BIT     0x40	/* X-interrupt mask */
#define M6811_H_BIT     0x20	/* Half carry flag */
#define M6811_I_BIT     0x10	/* I-interrupt mask */
#define M6811_N_BIT     0x08	/* Negative */
#define M6811_Z_BIT     0x04	/* Zero */
#define M6811_V_BIT     0x02	/* Overflow */
#define M6811_C_BIT     0x01	/* Carry */

/* 68HC11 register address offsets (range 0..0x3F or 0..64).
   The absolute address of the I/O register depends on the setting
   of the M6811_INIT register.  At init time, the I/O registers are
   mapped at 0x1000.  Address of registers is then:

   0x0000 + M6812_xxx
*/
#define M6812_PORTA     0x00	/* Port A register */
#define M6812_PORTB	0x01	/* Port B register */
#define M6812_DDRA      0x02	/* Data Direction Port A */
#define M6812_DDRB      0x03	/* Data Direction Port A */
#define M6812__RES4     0x04
#define M6812__RES5     0x05
#define M6812__RES6     0x06
#define M6812__RES7     0x07
#define M6812_PORTE     0x08    /* Port E register */
#define M6812_DDRE      0x09    /* DAta Direction Port E */
#define M6812_PEAR      0x0A
#define M6812_MODE      0x0B
#define M6812_PUCR      0x0C
#define M6812_RDRIV     0x0D    /* Reduced Driver Register */
#define M6812__RESE     0x0E
#define M6812__RESF     0x0F
#define M6812_INITRM	0x10	/* Ram mapping */
#define M6812_INITRG	0x11	/* Register mapping */
#define M6812_INITEE	0x12	/* Eeprom mapping */
#define M6812_MISC	0x13
#define M6812_RTICTL	0x14
#define M6812_RTIFLG	0x15
#define M6812_COPCTL	0x16    /* COP control register */
#define M6812_COPRST	0x17	/* COP reset register */
#define M6812_ITST0	0x18
#define M6812_ITST1	0x19
#define M6812_ITST2	0x1A
#define M6812_ITST3	0x1B
#define M6812__RES1C    0x1C
#define M6812__RES1D    0x1D
#define M6812_INTCR	0x1E
#define M6812_HPRIO	0x1F
#define M6812_BRKCT0	0x20
#define M6812_BRKCT1	0x21
#define M6812_BRKAH	0x22
#define M6812_BRKAL	0x23
#define M6812_BRKDH	0x24
#define M6812_BRKDL	0x25
#define M6812__RES26    0x26
#define M6812__RES27    0x27
#define M6812_PORTJ	0x28
#define M6812_PORTH	0x29
#define M6812_DDRJ	0x2A
#define M6812_DDRH	0x2B
#define M6812_KWIEJ	0x2C
#define M6812_KWIEH	0x2D
#define M6812_KWIFJ     0x2E
#define M6812_KWIFH     0x2F
#define M6812_KWPJ	0x30
#define M6812_KWPH	0x31
#define M6812__RES32	0x32
#define M6812__RES33	0x33
#define M6812__RES34	0x34
#define M6812__RES35	0x35
#define M6812__RES36	0x36
#define M6812__RES37	0x37
#define M6812_SYNR	0x38
#define M6812_REFDV     0x39
#define M6812_CGTFLG	0x3A
#define M6812_PLLFLG	0x3B
#define M6812_PLLCR	0x3C
#define M6812_CLKSEL	0x3D
#define M6812_SLOW	0x3E
#define M6812_CGTCTL	0x3F
#define M6812_PWCLK	0x40
#define M6812_PWPOL	0x41
#define M6812_PWEN	0x42
#define M6812_PWPRES	0x43
#define M6812_PWSCAL0	0x44
#define M6812_PWSCNT0	0x45
#define M6812_PWSCAL1	0x46
#define M6812_PWSCNT1	0x47
#define M6812_PWCNT0	0x48
#define M6812_PWCNT1	0x49
#define M6812_PWCNT2	0x4A
#define M6812_PWCNT3	0x4B
#define M6812_PWPER0	0x4C
#define M6812_PWPER1	0x4D
#define M6812_PWPER2	0x4E
#define M6812_PWPER3	0x4F
#define M6812_PWDTY0	0x50
#define M6812_PWCTL	0x51
#define M6812_PWTST	0x52
#define M6812_PORTP	0x53
#define M6812_DDRP	0x54

#define M6812_ATD0CTL0	0x60
#define M6812_ATD0CTL1	0x61
#define M6812_ATD0CTL2	0x62
#define M6812_ATD0CTL3	0x63
#define M6812_ATD0CTL4	0x64
#define M6812_ATD0CTL5	0x65
#define M6812_ATDOSTAT0 0x66
#define M6812_ATDOSTAT1 0x67
#define M6812_ATDOTESTH 0x68
#define M6812_ATDOTESTL 0x69

#define M6812_PORTAD0	0x6F
#define M6812_ADR00H	0x70
#define M6812_ADR00L	0x71
#define M6812_ADR01H	0x72
#define M6812_ADR01L	0x73
#define M6812_ADR02H	0x74
#define M6812_ADR02L	0x75
#define M6812_ADR03H	0x76
#define M6812_ADR03L	0x77
#define M6812_ADR04H	0x78
#define M6812_ADR04L	0x79
#define M6812_ADR05H	0x7A
#define M6812_ADR05L	0x7B
#define M6812_ADR06H	0x7C
#define M6812_ADR06L	0x7D
#define M6812_ADR07H	0x7E
#define M6812_ADR07L	0x7F
#define M6812_TIOS	0x80

#define M6812_CFORC	0x81
#define M6812_OC7M	0x82
#define M6812_OC7D	0x83
#define M6812_TCNT	0x84
#define M6812_TCNTH	0x84
#define M6812_TCNTL	0x85
#define M6812_TSCR	0x86
#define M6812_TQCR	0x87
#define M6812_TCTL1	0x88
#define M6812_TCTL2	0x89
#define M6812_TCTL3	0x8A
#define M6812_TCTL4	0x8B
#define M6812_TMSK1	0x8C
#define M6812_TMSK2	0x8D
#define M6812_TFLG1	0x8E
#define M6812_TFLG2	0x8F
#define M6812_TC0	0x90
#define M6812_TC1	0x92
#define M6812_TC2	0x94
#define M6812_TC3	0x96
#define M6812_TC4	0x98
#define M6812_TC5	0x9A
#define M6812_TC6	0x9C
#define M6812_TC7	0x9E
#define M6812_PACTL	0xA0
#define M6812_PAFLG	0xA1
#define M6812_PACN3	0xA2
#define M6812_PACN2	0xA3
#define M6812_PACN1	0xA4
#define M6812_PACN0	0xA5
#define M6812_MCCTL	0xA6
#define M6812_MCFLG	0xA7
#define M6812_ICPAR	0xA8
#define M6812_DLYCT	0xA9
#define M6812_ICOVW	0xAA
#define M6812_ICSYS	0xAB
#define M6812__RESAC	0xAC
#define M6812_TIMTST	0xAD
#define M6812_PORTT	0xAE
#define M6812_DDRT	0xAF
#define M6812_PBCTL	0xB0
#define M6812_PBFLG	0xB1
#define M6812_PA3H	0xB2
#define M6812_PA2H	0xB3
#define M6812_PA1H	0xB4
#define M6812_PA0H	0xB5
#define M6812_MCCNT	0xB6
#define M6812_TC0H	0xB8
#define M6812_TC1H	0xBA
#define M6812_TC2H	0xBC
#define M6812_TC3H	0xBE
#define M6812_SC0BD	0xC0
#define M6812_SC0BDH    0xC0
#define M6812_SC0BDL    0xC1
#define M6812_SC0CR1	0xC2
#define M6812_SC0CR2	0xC3
#define M6812_SC0SR1	0xC4
#define M6812_SC0SR2	0xC5
#define M6812_SC0DRH	0xC6
#define M6812_SC0DRL	0xC7
#define M6812_SC1BD	0xC8
#define M6812_SC1BDH	0xC8
#define M6812_SC1BDL	0xC9
#define M6812_SC1CR1	0xCA
#define M6812_SC1CR2	0xCB
#define M6812_SC1SR1    0xCC
#define M6812_SC1SR2    0xCD
#define M6812_SC1DRH	0xCE
#define M6812_SC1DRL	0xCF
#define M6812_SP0CR1	0xD0
#define M6812_SP0CR2	0xD1
#define M6812_SP0BR	0xD2
#define M6812_SP0SR	0xD3
#define M6812__RESD4	0xD4


/* Flags of the SCxCR1 register.  */
#define M6812_LOOPS	0x80	/* SCI Loop Mode/Single Wire Mode Enable */
#define M6812_WOMS      0x40	/* Wired-Or Mode for Serial Pins */
#define M6812_RSRC      0x20	/* Receiver source */
#define M6812_M		0x10	/* SCI Character length */
#define M6812_WAKE	0x08	/* Wake up method select (0=idle, 1=addr mark) */
#define M6812_ILT	0x04	/* Idle Line Type (0=short, 1=long) */
#define M6812_PE        0x02    /* Parity Enable */
#define M6812_PT        0x01    /* Parity Type (0=even, 1=odd) */

/* Flags of the SCxCR2 register.  */
#define M6812_TIE	0x80	/* Transmit Interrupt enable */
#define M6812_TCIE	0x40	/* Transmit Complete Interrupt Enable */
#define M6812_RIE	0x20	/* Receive Interrupt Enable */
#define M6812_ILIE	0x10	/* Idle Line Interrupt Enable */
#define M6812_TE	0x08	/* Transmit Enable */
#define M6812_RE	0x04	/* Receive Enable */
#define M6812_RWU	0x02	/* Receiver Wake Up */
#define M6812_SBK	0x01	/* Send Break */

/* Flags of the SCxSR1 register.  */
#define M6812_TDRE	0x80	/* Transmit Data Register Empty */
#define M6812_TC	0x40	/* Transmit Complete */
#define M6812_RDRF	0x20	/* Receive Data Register Full */
#define M6812_IDLE	0x10	/* Idle Line Detect */
#define M6812_OR	0x08	/* Overrun Error */
#define M6812_NF	0x04	/* Noise Flag */
#define M6812_FE	0x02	/* Framing Error */
#define M6812_PF	0x01	/* Parity Error flag */

/* Flags of the SCxSR2 register.  */
#define M6812_RAF	0x01	/* Receiver Active Flag */

/* Flags in the SCxDRH register */
#define M6812_R8        0x80    /* Receive Bit 8 */
#define M6812_T8        0x40    /* Transmit bit 8 */


#define M6812_IO_SIZE   (0x3ff)

/* The I/O registers are represented by a volatile array.
   Address if fixed at link time.  */
extern volatile unsigned char _io_ports[];

#endif /* _M68HC12_PORTS_DEF_H */

