/* Small Calculator example program
   Copyright (C) 1999, 2000 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)	

This file is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

In addition to the permissions in the GNU General Public License, the
Free Software Foundation gives you unlimited permission to link the
compiled version of this file with other programs, and to distribute
those programs without any restriction coming from the use of this
file.  (The General Public License restrictions do apply in other
respects; for example, they cover modification of the file, and
distribution when not linked into another program.)

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#ifndef _CALC_H
#define _CALC_H

#define MAX_VALUES 32

#if !defined(VALUE_16) && !defined(VALUE_32) && !defined(VALUE_64)
# define VALUE_16
#endif

/* Select the size of integers used by the calculator.  */
#ifdef VALUE_16
typedef short value_t;
#define HEX_CVT_MASK 0x0fff
#elif defined(VALUE_32)
typedef long value_t;
#define HEX_CVT_MASK 0x0fffffffL
#elif defined(VALUE_64)
typedef long long value_t;
#define HEX_CVT_MASK 0x0fffffffffffffffLL
#endif

/* List of operations/commands supported by the tool.  */
typedef enum op_type 
{
  OP_ADD,
  OP_SUB,
  OP_MUL,
  OP_DIV,
  OP_MOD,
  OP_AND,
  OP_OR,
  OP_XOR,
  OP_NOT,
  OP_NEG,
  OP_QUIT,
  OP_LIST,
  OP_NUMBER,
  OP_DEC,
  OP_HEX,
  OP_HELP
} op_type;

typedef struct command
{
  const char* name;
  op_type type;
  const char* help;
} command;

/* Definition of the format that we must use to print the value.  */
typedef enum print_mode
{
  PRINT_DEC,
  PRINT_HEX,
  PRINT_OCT,
  PRINT_BIN
} print_mode;

/* The stack of values.  The operations operate on the stack.  They pop
   one or two values and push the result on the stack (like some HP
   calculators).  */
typedef struct value_stack 
{
  value_t* values;
  int top;
  int max;
  print_mode mode;
} value_stack_t;

/* Stack methods.  */
extern int     push_value  (value_stack_t* stack, value_t value);
extern value_t pop_value   (value_stack_t* stack);
extern int     operation   (value_stack_t* stack, enum op_type op);
extern void    print_value (value_stack_t* stack, print_mode mode, int which);
extern int     calc_loop   (value_stack_t* stack);


#ifdef __linux__
inline void
serial_print (char* p)
{
  write(1, p, strlen(p));
}
inline int
serial_recv ()
{
  char c;
  read (0, &c, 1);
  return c;
}
inline int
serial_init ()
{
  return 0;
}
#else

#include <m68hc11/sio.h>

inline void
print (const char* msg)
{
  serial_print (msg);
}

#endif

#endif
