/* Trap example for 68HC11
   Copyright (C) 1999 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)	

This file is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

In addition to the permissions in the GNU General Public License, the
Free Software Foundation gives you unlimited permission to link the
compiled version of this file with other programs, and to distribute
those programs without any restriction coming from the use of this
file.  (The General Public License restrictions do apply in other
respects; for example, they cover modification of the file, and
distribution when not linked into another program.)

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "trap.h"

extern int global_result;

int
main()
{
  int result;
  int failed;
  _serial_init ();

  /* Test with simple trap handler.  */
  set_trap_handler ((handler) simple_trap_handler);
  print ("Using simple trap handler: ");
  failed = 0;
  simple_trap_handler (1);
  if (global_result != 1)
    {
      print ("  Simple trap handler failed, didn't returned 1\r\n");
      failed = 1;
    }
  
  simple_trap_handler (2);
  if (global_result != 4)
    {
      print ("  Simple trap handler failed, didn't returned 4\r\n");
      failed = 1;
    }
  if (failed == 0)
    print ("OK\r\n");

  /* Test with add trap handler.  */
  set_trap_handler ((handler) add_trap_handler);
  failed = 0;
  print ("Using add trap handler: ");
  result = add_trap_handler (1, 2, 3, 4);
  if (result != 10)
    {
      print ("Add trap handler didn't returned 10\r\n");
      failed = 1;
    }
  result = add_trap_handler (5, 6, 7, 8);
  if (result != 26)
    {
      print ("Add trap handler didn't returned 26\r\n");
      failed = 1;
    }
  if (failed == 0)
    print ("OK\r\n");

  set_trap_handler ((handler) os_trap_handler);
  failed = 0;
  print ("Using OS trap handler...\r\n");
  result = os_trap_handler (0, "  Hello World from os_trap_handler...\r\n");
  if (result != 0)
    {
      print ("  OS call 0 failed\r\n");
      failed = 1;
    }

  result = os_trap_handler (0, "  Type a character ");
  if (result != 0)
    {
      print ("  OS call 0 failed\r\n");
      failed = 1;
    }
  
  result = os_trap_handler (1);

  result = os_trap_handler (2, 23, 44);
  if (result != 23 + 44)
    {
      print ("  OS add system call failed\r\n");
      failed = 1;
    }
  
  if (failed == 0)
    print ("OS trap call test ok.\r\n");
  
  print ("Trap handler test finished.\r\n");
  return 0;
}

